{{- define "lambdalayer-utils.serviceAccountUtilityName" -}}
{{ .Release.Name }}-{{ .Chart.Name }}-sa
{{- end -}}

{{- define "lambdalayer-utils.serviceAccountUtilityRoleName" -}}
{{ .Release.Name }}-{{ .Chart.Name }}-role
{{- end -}}

{{- define "lambdalayer-utils.serviceAccountUtilityRoleBindingName" -}}
{{ .Release.Name }}-{{ .Chart.Name }}-rb
{{- end -}}

{{- define "lambdalayer-utils.imagePullSecrets" -}}
{{ range $secrets := .Values.global.imagePullSecrets }}
- name: {{ $secrets }}
{{ end }}
{{- end -}}

{{- define "lambdalayer-utils.serviceAccountUtility" -}}
apiVersion: v1
kind: ServiceAccount
metadata:
  name: {{ include "lambdalayer-utils.serviceAccountUtilityName" . }}
  namespace: {{ include "lambdalayer-utils.appsNamespace" . }}
  labels:
    {{- include "lambdalayer-utils.labels" . | nindent 4 }}
  annotations:
    {{- include "lambdalayer-utils.preInstallUpgradeHook" (dict "root" . "weight" 0) | nindent 4 }}
    {{- include "lambdalayer-utils.annotations" . | nindent 4 }}
{{ if .Values.global.addImagePullSecret }}
imagePullSecrets:
{{ include "lambdalayer-utils.imagePullSecrets" . }}
{{ end }}

---
apiVersion: v1
kind: ServiceAccount
metadata:
  name: {{ include "lambdalayer-utils.serviceAccountUtilityName" . }}
  namespace: {{ include "lambdalayer-utils.infraNamespace" . }}
  labels:
    {{- include "lambdalayer-utils.labels" . | nindent 4 }}
  annotations:
    {{- include "lambdalayer-utils.preInstallUpgradeHook" (dict "root" . "weight" 0) | nindent 4 }}
    {{- include "lambdalayer-utils.annotations" . | nindent 4 }}
{{ if .Values.global.addImagePullSecret }}
imagePullSecrets:
{{ include "lambdalayer-utils.imagePullSecrets" . }}
{{ end }}

---
apiVersion: rbac.authorization.k8s.io/v1
kind: Role
metadata:
  namespace: {{ include "lambdalayer-utils.appsNamespace" . }}
  name: {{ include "lambdalayer-utils.serviceAccountUtilityRoleName" . }}
  labels:
    {{- include "lambdalayer-utils.labels" . | nindent 4 }}
  annotations:
    {{- include "lambdalayer-utils.preInstallUpgradeHook" (dict "root" . "weight" 0) | nindent 4 }}
    {{- include "lambdalayer-utils.annotations" . | nindent 4 }}
rules:
- apiGroups: ["batch"]
  resources: ["jobs"]
  verbs: ["get", "list", "watch", "create", "update", "patch", "delete"]
- apiGroups: [""]
  resources: ["configmaps", "secrets", "services", "pods"]
  verbs: ["get", "list", "watch", "create", "update", "patch", "delete"]
- apiGroups: ["extensions", "apps"]
  resources: ["deployments"]
  verbs: ["get", "list", "watch", "create", "update", "patch", "delete"]

---
apiVersion: rbac.authorization.k8s.io/v1
kind: Role
metadata:
  namespace: {{ include "lambdalayer-utils.infraNamespace" . }}
  name: {{ include "lambdalayer-utils.serviceAccountUtilityRoleName" . }}
  labels:
    {{- include "lambdalayer-utils.labels" . | nindent 4 }}
  annotations:
    {{- include "lambdalayer-utils.preInstallUpgradeHook" (dict "root" . "weight" 0) | nindent 4 }}
    {{- include "lambdalayer-utils.annotations" . | nindent 4 }}
rules:
- apiGroups: ["batch"]
  resources: ["jobs"]
  verbs: ["get", "list", "watch", "create", "update", "patch", "delete"]
- apiGroups: [""]
  resources: ["configmaps", "secrets", "services", "pods"]
  verbs: ["get", "list", "watch", "create", "update", "patch", "delete"]
- apiGroups: ["extensions", "apps"]
  resources: ["deployments"]
  verbs: ["get", "list", "watch", "create", "update", "patch", "delete"]

---
# Allow access into apps namespace from infra
apiVersion: rbac.authorization.k8s.io/v1
kind: RoleBinding
metadata:
  name: {{ include "lambdalayer-utils.serviceAccountUtilityRoleBindingName" . }}
  namespace: {{ include "lambdalayer-utils.appsNamespace" . }}
  labels:
    {{- include "lambdalayer-utils.labels" . | nindent 4 }}
  annotations:
    {{- include "lambdalayer-utils.preInstallUpgradeHook" (dict "root" . "weight" 1) | nindent 4 }}
    {{- include "lambdalayer-utils.annotations" . | nindent 4 }}
subjects:
- kind: ServiceAccount
  name: {{ include "lambdalayer-utils.serviceAccountUtilityName" . }}
  namespace: {{ include "lambdalayer-utils.infraNamespace" . }}
roleRef:
  kind: Role
  name: {{ include "lambdalayer-utils.serviceAccountUtilityRoleName" . }}
  apiGroup: rbac.authorization.k8s.io

---
# Allow infra to manage infra namespace
apiVersion: rbac.authorization.k8s.io/v1
kind: RoleBinding
metadata:
  name: {{ include "lambdalayer-utils.serviceAccountUtilityRoleBindingName" . }}
  namespace: {{ include "lambdalayer-utils.infraNamespace" . }}
  labels:
    {{- include "lambdalayer-utils.labels" . | nindent 4 }}
  annotations:
    {{- include "lambdalayer-utils.preInstallUpgradeHook" (dict "root" . "weight" 1) | nindent 4 }}
    {{- include "lambdalayer-utils.annotations" . | nindent 4 }}
subjects:
- kind: ServiceAccount
  name: {{ include "lambdalayer-utils.serviceAccountUtilityName" . }}
  namespace: {{ include "lambdalayer-utils.infraNamespace" . }}
roleRef:
  kind: Role
  name: {{ include "lambdalayer-utils.serviceAccountUtilityRoleName" . }}
  apiGroup: rbac.authorization.k8s.io
{{- end -}}
